import datetime
from typing import Any, Dict, List, Optional

from yamcs.core.helpers import parse_server_time
from yamcs.protobuf.events import events_pb2
from yamcs.protobuf.instances import instances_pb2
from yamcs.protobuf.services import services_pb2


class AuthInfo:
    """
    Authentication information
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def require_authentication(self) -> bool:
        return self._proto.requireAuthentication


class ServerInfo:
    """
    General server properties.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def id(self) -> str:
        """The Server ID."""
        return self._proto.serverId

    @property
    def version(self) -> str:
        """The version of Yamcs Server."""
        return self._proto.yamcsVersion

    @property
    def default_yamcs_instance(self) -> Optional[str]:
        """Returns the default Yamcs instance."""
        if self._proto.HasField("defaultYamcsInstance"):
            return self._proto.defaultYamcsInstance
        return None

    def __str__(self):
        return f"{self.id} (v{self.version})"


class UserInfo:
    """
    Info on a Yamcs User.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def username(self) -> str:
        return self._proto.name

    @property
    def superuser(self) -> bool:
        return self._proto.superuser

    @property
    def system_privileges(self) -> List[str]:
        return [p for p in self._proto.systemPrivileges]

    @property
    def object_privileges(self) -> List["ObjectPrivilege"]:
        return [ObjectPrivilege(p) for p in self._proto.objectPrivileges]

    def __str__(self):
        return self.username


class ObjectPrivilege:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        return self._proto.type

    @property
    def objects(self) -> List[str]:
        return [o for o in self._proto.objects]

    def __str__(self):
        return self.name


class Event:
    """
    A timetagged free-text message. Events work a lot like log
    messages in logging frameworks, but then targeted at operators.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def generation_time(self) -> datetime.datetime:
        """
        The time when the event was generated.
        """
        return parse_server_time(self._proto.generationTime)

    @property
    def reception_time(self) -> datetime.datetime:
        """
        The time when the event was received by Yamcs.
        """
        return parse_server_time(self._proto.receptionTime)

    @property
    def severity(self) -> Optional[str]:
        """
        Severity level of the event. One of ``INFO``, ``WATCH``,
        ``WARNING``, ``DISTRESS``, ``CRITICAL`` or ``SEVERE``.
        """
        if self._proto.HasField("severity"):
            return events_pb2.Event.EventSeverity.Name(self._proto.severity)
        return None

    @property
    def message(self) -> Optional[str]:
        """
        Event message.
        """
        if self._proto.HasField("message"):
            return self._proto.message
        return None

    @property
    def sequence_number(self) -> int:
        """
        Sequence number. Usually this is assigned by the source of the event.
        """
        return self._proto.seqNumber

    @property
    def event_type(self) -> Optional[str]:
        """
        The event type. This is mission-specific and can be any string.
        """
        if self._proto.HasField("type"):
            return self._proto.type
        return None

    @property
    def source(self) -> Optional[str]:
        """
        The event source. Can be any string.
        """
        if self._proto.HasField("source"):
            return self._proto.source
        return None

    @property
    def extra(self) -> Dict[str, str]:
        """
        Dict with extra event properties.

        .. versionadded:: 1.8.4
           Compatible with Yamcs 5.7.3 onwards
        """
        return {key: self._proto.extra[key] for key in self._proto.extra}

    def __str__(self):
        return f"{self.generation_time} [{self.severity}] {self.message}"


class Link:
    """
    Represents a link with an external system. Depending on the
    semantics of the link, this may imply inbound data, outbound
    data or a combination of both.
    """

    def __init__(self, proto):
        self._proto = proto
        self._actions = [LinkAction(action) for action in self._proto.actions]
        self._extra = {key: value for key, value in proto.extra.items()}

    @property
    def instance(self) -> str:
        """Name of the instance where this link is defined."""
        return self._proto.instance

    @property
    def name(self) -> str:
        """Name of this link (unique per instance)."""
        return self._proto.name

    @property
    def class_name(self) -> str:
        """Name of this link's class."""
        return self._proto.type

    @property
    def enabled(self) -> bool:
        """If ``True``, this link accepts or outputs data."""
        return not self._proto.disabled

    @property
    def status(self) -> str:
        """Short status."""
        return self._proto.status

    @property
    def in_count(self) -> int:
        """The number of inbound data events (example: packet count)."""
        return self._proto.dataInCount

    @property
    def out_count(self) -> int:
        """The number of outbound data events (example: command count)."""
        return self._proto.dataOutCount

    @property
    def actions(self) -> List["LinkAction"]:
        """Custom actions."""
        return self._actions

    @property
    def extra(self) -> Dict[str, Any]:
        """Custom info fields."""
        return self._extra

    def __str__(self):
        desc = f"{self.instance}/{self.name}"
        return f"{desc}: {self.status} (in: {self.in_count} out: {self.out_count})"


class LinkAction:
    def __init__(self, proto):
        self._proto = proto

    def __str__(self):
        return str(self._proto)

    @property
    def id(self) -> str:
        """Action ID"""
        return self._proto.id

    @property
    def label(self) -> str:
        """Label for the action"""
        return self._proto.label

    @property
    def style(self) -> str:
        """Action style"""
        return self._proto.style

    @property
    def enabled(self) -> bool:
        """Whether the action is currently enabled"""
        return self._proto.enabled

    @property
    def checked(self) -> bool:
        """Whether the action is currently checked"""
        return self._proto.checked


class RdbTablespace:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        """Tablespace name"""
        return self._proto.name

    @property
    def data_dir(self) -> str:
        """Data directory"""
        return self._proto.dataDir


class Instance:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        """Name of this instance."""
        return self._proto.name

    @property
    def state(self) -> str:
        """
        State of this instance. One of ``OFFLINE``, ``INITIALIZING``,
        ``INITIALIZED``, ``STARTING``, ``RUNNING``, ``STOPPING`` or
        ``FAILED``.
        """
        return instances_pb2.YamcsInstance.InstanceState.Name(self._proto.state)

    @property
    def failure_cause(self) -> Optional[str]:
        """Failure message when ``state == 'FAILED'``"""
        if self._proto.HasField("failureCause"):
            return self._proto.failureCause
        return None

    @property
    def mission_time(self) -> Optional[datetime.datetime]:
        """
        Mission time of this instance's time service.
        """
        if self._proto.HasField("missionTime"):
            return parse_server_time(self._proto.missionTime)
        return None

    def __str__(self):
        return f"{self.name} [{self.state}]"


class InstanceTemplate:
    """A template for creating an instance."""

    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        """Name of this template."""
        return self._proto.name

    def __str__(self):
        return self.name


class Service:
    """A Yamcs service."""

    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        """Name of this service."""
        return self._proto.name

    @property
    def instance(self) -> Optional[str]:
        """Name of the instance where this service is defined."""
        if self._proto.HasField("instance"):
            return self._proto.instance
        return None

    @property
    def processor(self) -> Optional[str]:
        """Name of the processor where this service is defined."""
        if self._proto.HasField("processor"):
            return self._proto.processor
        return None

    @property
    def class_name(self) -> str:
        """Name of this service's class."""
        return self._proto.className

    @property
    def state(self) -> str:
        """State of this service."""
        return services_pb2.ServiceState.Name(self._proto.state)

    def failure_message(self) -> Optional[str]:
        """
        Short failure message when state is ``FAILED``

        .. versionadded:: 1.9.0
           Compatible with Yamcs 5.8.0 onwards
        """
        if self._proto.HasField("failureMessage"):
            return self._proto.failureMessage
        return None

    def failure_cause(self) -> Optional[str]:
        """
        Java stacktrace when state is ``FAILED``

        .. versionadded:: 1.9.0
           Compatible with Yamcs 5.8.0 onwards
        """
        if self._proto.HasField("failureCause"):
            return self._proto.failureCause
        return None

    def __str__(self):
        return f"{self.name} [{self.state}]"


class Processor:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self) -> str:
        """Name of this processor."""
        return self._proto.name

    @property
    def instance(self) -> str:
        """Name of the instance where this processor is defined."""
        return self._proto.instance

    @property
    def state(self) -> str:
        """State of this processor."""
        return services_pb2.ServiceState.Name(self._proto.state)

    @property
    def type(self) -> str:
        """Type of this processor."""
        return self._proto.type

    @property
    def owner(self) -> str:
        """User that owns this processor."""
        return self._proto.creator

    @property
    def persistent(self) -> bool:
        """If ``True``, this processor does not close if no clients are connected."""
        return self._proto.persistent

    @property
    def protected(self) -> bool:
        """If ``True``, this processor can not be deleted."""
        return self._proto.protected

    @property
    def mission_time(self) -> Optional[datetime.datetime]:
        """
        Mission time of this processor.
        """
        if self._proto.HasField("time"):
            return parse_server_time(self._proto.time)
        return None

    def __str__(self):
        return f"{self.name} [{self.state}]"


class LoadParameterValuesResult:
    """
    Statistics returned when loading a stream of
    parameter values.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def value_count(self) -> str:
        """Number of loaded parameter values."""
        return self._proto.valueCount

    @property
    def min_generation_time(self) -> Optional[datetime.datetime]:
        """Minimum geneneration time of all loaded parameter values"""
        if self._proto.HasField("minGenerationTime"):
            return parse_server_time(self._proto.minGenerationTime)
        return None

    @property
    def max_generation_time(self) -> Optional[datetime.datetime]:
        """Maximum geneneration time of all loaded parameter values"""
        if self._proto.HasField("maxGenerationTime"):
            return parse_server_time(self._proto.maxGenerationTime)
        return None

    def __str__(self) -> str:
        result = f"{self.value_count} values"
        if self.max_generation_time:
            result += (
                f" in range [{self.min_generation_time} - {self.max_generation_time}]"
            )
        return result



class YamcsError(Exception):
    """Base class for raised exceptions."""


class ConnectionFailure(YamcsError):
    """Yamcs is not or no longer available."""


class TimeoutError(YamcsError):
    """The operation exceeded the given deadline."""


class NotFound(YamcsError):
    """The resource was not found."""


class Unauthorized(YamcsError):
    """Unable to get access the resource."""



from datetime import datetime, timezone
from typing import Callable, Optional, Union

import requests
import urllib3
from google.protobuf.message import DecodeError
from yamcs import clientversion
from yamcs.api import exception_pb2
from yamcs.core.auth import Credentials
from yamcs.core.exceptions import NotFound, Unauthorized, YamcsError
from yamcs.core.helpers import FixedDelay, do_request


class Context:

    def __init__(
        self,
        address: str,
        tls: bool = False,
        credentials: Optional[Credentials] = None,
        user_agent: Optional[str] = None,
        on_token_update: Optional[Callable[[Credentials], None]] = None,
        tls_verify: Union[bool, str] = True,
        keep_alive: bool = True,
    ):
        if address.endswith("/"):
            self.address = address[:-1]
        else:
            self.address = address

        if tls:
            self.url = f"https://{self.address}"
            self.auth_root = f"https://{self.address}/auth"
            self.api_root = f"https://{self.address}/api"
            self.ws_root = f"wss://{self.address}/api/websocket"
        else:
            self.url = f"http://{self.address}"
            self.auth_root = f"http://{self.address}/auth"
            self.api_root = f"http://{self.address}/api"
            self.ws_root = f"ws://{self.address}/api/websocket"

        self.session = requests.Session()
        self.session.verify = tls_verify
        if not tls_verify:
            try:
                # requests < 2.16.0
                requests.packages.urllib3.disable_warnings(
                    requests.packages.urllib3.exceptions.InsecureRequestWarning
                )
            except Exception:
                # requests >= 2.16.0
                urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        self.credentials: Optional[Credentials] = None
        self._session_renewer = None

        if credentials:
            converted_creds = credentials.login(
                self.session, self.auth_root, on_token_update
            )
            self.credentials = converted_creds

            # An assigned refresh token lives only for about 30 minutes. We actively
            # extend it, so that the session survives when idle.
            if converted_creds.expiry and keep_alive:

                def renew_session():
                    expiry = converted_creds.expiry
                    if expiry:
                        remaining = expiry - datetime.now(tz=timezone.utc)
                        if 0 < remaining.total_seconds() < 60:
                            converted_creds.refresh(self.session, self.auth_root)

                self._session_renewer = FixedDelay(renew_session, 10)

        if not user_agent:
            user_agent = "python-yamcs-client v" + clientversion.__version__
        self.session.headers.update({"User-Agent": user_agent})

    def get_proto(self, path: str, **kwargs) -> requests.Response:
        headers = kwargs.pop("headers", {})
        headers["Accept"] = "application/protobuf"
        kwargs.update({"headers": headers})
        return self.request("get", path, **kwargs)

    def put_proto(self, path: str, **kwargs) -> requests.Response:
        headers = kwargs.pop("headers", {})
        headers["Content-Type"] = "application/protobuf"
        headers["Accept"] = "application/protobuf"
        kwargs.update({"headers": headers})
        return self.request("put", path, **kwargs)

    def patch_proto(self, path: str, **kwargs) -> requests.Response:
        headers = kwargs.pop("headers", {})
        headers["Content-Type"] = "application/protobuf"
        headers["Accept"] = "application/protobuf"
        kwargs.update({"headers": headers})
        return self.request("patch", path, **kwargs)

    def post_proto(self, path: str, **kwargs) -> requests.Response:
        headers = kwargs.pop("headers", {})
        headers["Content-Type"] = "application/protobuf"
        headers["Accept"] = "application/protobuf"
        kwargs.update({"headers": headers})
        return self.request("post", path, **kwargs)

    def delete_proto(self, path: str, **kwargs) -> requests.Response:
        headers = kwargs.pop("headers", {})
        headers["Accept"] = "application/protobuf"
        kwargs.update({"headers": headers})
        return self.request("delete", path, **kwargs)

    def request(self, method: str, path: str, **kwargs) -> requests.Response:
        path = f"{self.api_root}{path}"

        if self.credentials:
            self.credentials.before_request(self.session, self.auth_root)

        response = do_request(self.session, method, path, **kwargs)
        if 200 <= response.status_code < 300:
            return response

        exception_message = exception_pb2.ExceptionMessage()
        try:
            exception_message.ParseFromString(response.content)
        except DecodeError:
            pass

        if response.status_code == 401:
            raise Unauthorized("401 Client Error: Unauthorized")
        elif response.status_code == 404:
            msg = getattr(exception_message, "msg")
            raise NotFound(f"404 Client Error: {msg}")
        elif 400 <= response.status_code < 500:
            msg = getattr(exception_message, "msg")
            raise YamcsError(f"{response.status_code} Client Error: {msg}")
        msg = getattr(exception_message, "msg")
        raise YamcsError(f"{response.status_code} Server Error: {msg}")

    def close(self):
        """Close this context"""

        if self._session_renewer:
            self._session_renewer.stop()

        self.session.close()



from dataclasses import dataclass, field
from typing import Any, List, Mapping, Optional, Union

from yamcs.core.helpers import to_argument_value
from yamcs.protobuf.activities import activities_pb2


@dataclass
class Activity:
    """
    Superclass for activities. Implementations:

    * :class:`.CommandActivity`
    * :class:`.CommandStackActivity`
    * :class:`.ManualActivity`
    * :class:`.ScriptActivity`
    """

    @staticmethod
    def _as_subclass(proto):
        # No need for MANUAL.
        # It is recognized by Yamcs, by not having an
        # activity definition.
        if proto.type == "COMMAND":
            return CommandActivity._from_proto(proto)
        elif proto.type == "COMMAND_STACK":
            return CommandStackActivity._from_proto(proto)
        elif proto.type == "SCRIPT":
            return ScriptActivity._from_proto(proto)
        else:
            raise ValueError("Unexpected activity type")


@dataclass
class ManualActivity(Activity):
    """
    An activity whose execution status is managed outside of Yamcs
    """


@dataclass
class ScriptActivity(Activity):
    """
    An activity that runs a script
    """

    script: str
    """
    Script identifier.

    This should be the relative path to an an executable file in one of
    the search locations. When unconfigured, the default search
    location is :file:`etc/scripts/` relative to the Yamcs working
    directory.
    """

    args: Optional[Union[str, List[str]]] = None
    """
    Optional script arguments, passed verbatim in the command line.
    """

    processor: Optional[str] = None
    """
    Optional processor name. If provided, this is provided to the
    script as the environment variable ``YAMCS_PROCESSOR``.
    """

    @staticmethod
    def _from_proto(proto: activities_pb2.ActivityDefinitionInfo):
        activity = ScriptActivity(script=proto.args["script"])

        if "args" in proto.args:
            activity.args = proto.args["args"]

        if "processor" in proto.args:
            activity.processor = proto.args["processor"]

        return activity

    def _to_proto(self) -> activities_pb2.ActivityDefinitionInfo:
        proto = activities_pb2.ActivityDefinitionInfo()
        proto.type = "SCRIPT"
        proto.args["script"] = self.script

        if self.args:
            proto.args["args"] = self.args

        if self.processor:
            proto.args["processor"] = self.processor

        return proto


@dataclass
class CommandActivity(Activity):
    """
    An activity that executes a single command
    """

    command: str
    """Qualified name of a command"""

    args: Mapping[str, Any] = field(default_factory=dict)
    """Named arguments, if the command requires any"""

    extra: Mapping[str, Any] = field(default_factory=dict)
    """Extra command options"""

    processor: Optional[str] = None
    """
    Optional processor name. If not provided, Yamcs defaults to any
    processor it can find with commanding enabled.
    """

    @staticmethod
    def _from_proto(proto: activities_pb2.ActivityDefinitionInfo):
        activity = CommandActivity(command=proto.args["command"])

        if "args" in proto.args:
            command_args = proto.args["args"]
            activity.args = {key: value for key, value in command_args.items()}

        if "extra" in proto.args:
            command_extra = proto.args["extra"]
            activity.extra = {key: value for key, value in command_extra.items()}

        if "processor" in proto.args:
            activity.processor = proto.args["processor"]

        return activity

    def _to_proto(self) -> activities_pb2.ActivityDefinitionInfo:
        proto = activities_pb2.ActivityDefinitionInfo()
        proto.type = "COMMAND"
        proto.args["command"] = self.command

        if self.processor:
            proto.args["processor"] = self.processor

        proto.args["args"] = {}
        for k, v in self.args.items():
            proto.args["args"][k] = to_argument_value(v, force_string=True)

        proto.args["extra"] = {}
        for k, v in self.extra.items():
            proto.args["extra"][k] = to_argument_value(v, force_string=False)

        return proto


@dataclass
class CommandStackActivity(Activity):
    """
    An activity that executes a command stack
    """

    bucket: str
    """The name of the bucket containing the stack."""

    stack: str
    """The name of the stack object inside the bucket."""

    processor: Optional[str] = None
    """
    Optional processor name. If not provided, Yamcs defaults to any
    processor it can find with commanding enabled.
    """

    @staticmethod
    def _from_proto(proto: activities_pb2.ActivityDefinitionInfo):
        activity = CommandStackActivity(
            bucket=proto.args["bucket"],
            stack=proto.args["stack"],
        )

        if "processor" in proto.args:
            activity.processor = proto.args["processor"]

        return activity

    def _to_proto(self) -> activities_pb2.ActivityDefinitionInfo:
        proto = activities_pb2.ActivityDefinitionInfo()
        proto.type = "COMMAND_STACK"
        proto.args["bucket"] = self.bucket
        proto.args["stack"] = self.stack
        if self.processor:
            proto.args["processor"] = self.processor
        return proto



import functools
from typing import Any, Callable, Dict, Mapping, Optional

from google.protobuf import json_format, struct_pb2
from yamcs.core.context import Context
from yamcs.core.futures import WebSocketSubscriptionFuture
from yamcs.core.subscriptions import WebSocketSubscriptionManager
from yamcs.link.model import Cop1Config, Cop1Status
from yamcs.model import Link
from yamcs.protobuf.cop1 import cop1_pb2
from yamcs.protobuf.links import links_pb2


def _wrap_callback_parse_cop1_status(subscription, on_data, message):
    """
    Wraps a user callback to parse Cop1Status
    from a WebSocket data message
    """
    pb = cop1_pb2.Cop1Status()
    message.Unpack(pb)
    cop1_status = Cop1Status(pb)
    subscription._process(cop1_status)
    if on_data:
        on_data(cop1_status)


class Cop1Subscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to COP1 status updates.
    """

    def __init__(self, manager):
        super(Cop1Subscription, self).__init__(manager)
        self._status = None

    def get_status(self) -> Optional[Cop1Status]:
        """
        Returns the latest known COP1 status.
        """
        return self._status

    @property
    def cop1_active(self) -> Optional[bool]:
        if self._status:
            return self._status.cop1_active
        return None

    @property
    def state(self) -> Optional[str]:
        if self._status:
            return self._status.state
        return None

    @property
    def bypass_all(self) -> Optional[bool]:
        if self._status:
            return self._status.bypass_all
        return None

    @property
    def v_s(self) -> Optional[int]:
        if self._status:
            return self._status.v_s
        return None

    @property
    def nn_r(self) -> Optional[int]:
        if self._status:
            return self._status.nn_r
        return None

    def _process(self, update):
        self._status = update


class LinkClient:
    """Client object that groups operations for a specific link."""

    def __init__(self, ctx: Context, instance: str, link: str):
        super(LinkClient, self).__init__()
        self.ctx = ctx
        self._instance = instance
        self._link = link

    def get_info(self) -> Link:
        """
        Get info on this link.
        """
        response = self.ctx.get_proto(f"/links/{self._instance}/{self._link}")
        message = links_pb2.LinkInfo()
        message.ParseFromString(response.content)
        return Link(message)

    def enable_link(self):
        """
        Enables this link.
        """
        req = links_pb2.EnableLinkRequest()
        url = f"/links/{self._instance}/{self._link}:enable"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def disable_link(self):
        """
        Disables this link.
        """
        req = links_pb2.DisableLinkRequest()
        url = f"/links/{self._instance}/{self._link}:disable"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def run_action(
        self,
        action: str,
        message: Optional[Mapping[str, Any]] = None,
    ) -> Dict[str, Any]:
        """
        Runs the given action for this link

        :param action:
            action identifier
        :param message:
            action message
        :return:
            Action result (if the action returns anything)
        """
        req = links_pb2.RunActionRequest()
        if message:
            req.message.update(message)

        url = f"/links/{self._instance}/{self._link}/actions/{action}"
        response = self.ctx.post_proto(url, data=req.message.SerializeToString())
        response_message = struct_pb2.Struct()
        response_message.ParseFromString(response.content)
        return json_format.MessageToDict(response_message)

    def get_cop1_config(self) -> Cop1Config:
        """
        Gets the COP1 configuration for a data link.
        """
        response = self.ctx.get_proto(f"/cop1/{self._instance}/{self._link}/config")
        message = cop1_pb2.Cop1Config()
        message.ParseFromString(response.content)
        return Cop1Config(message)

    def update_cop1_config(
        self,
        window_width: Optional[int] = None,
        timeout_type: Optional[str] = None,
        tx_limit: Optional[int] = None,
        t1: Optional[float] = None,
    ) -> Cop1Config:
        """
        Sets the COP1 configuration for a data link.
        """
        req = cop1_pb2.Cop1Config()
        if window_width is not None:
            req.windowWidth = window_width
        if timeout_type is not None:
            req.timeoutType = cop1_pb2.TimeoutType.Value(timeout_type)
        if tx_limit is not None:
            req.txLimit = tx_limit
        if t1 is not None:
            req.t1 = int(round(1000 * t1))

        url = f"/cop1/{self._instance}/{self._link}/config"
        response = self.ctx.patch_proto(url, data=req.SerializeToString())

        message = cop1_pb2.Cop1Config()
        message.ParseFromString(response.content)
        return Cop1Config(message)

    def disable_cop1(self, bypass_all: bool = True):
        """
        Disable COP1 for a data link.

        :param bypass_all:
            All frames have bypass activated (i.e. they will be BD frames)
        """
        req = cop1_pb2.DisableRequest()
        req.setBypassAll = bypass_all
        url = f"/cop1/{self._instance}/{self._link}:disable"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def initialize_cop1(
        self,
        type: str,
        clcw_wait_timeout: Optional[int] = None,
        v_r: Optional[int] = None,
    ):
        """
        Initialize COP1.

        :param type:
            One of ``WITH_CLCW_CHECK``, ``WITHOUT_CLCW_CHECK``,
            ``UNLOCK`` or ``SET_VR``
        :param clcw_wait_timeout:
            timeout in seconds used for the reception of
            CLCW. Required if type is ``WITH_CLCW_CHECK``
        :param v_r:
            value of v(R) if type is set to ``SET_VR``
        """
        req = cop1_pb2.InitializeRequest()
        req.type = cop1_pb2.InitializationType.Value(type)

        if clcw_wait_timeout is not None:
            req.clcwCheckInitializeTimeout = int(1000 * clcw_wait_timeout)
        if v_r is not None:
            req.vR = v_r

        url = f"/cop1/{self._instance}/{self._link}:initialize"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def resume_cop1(self):
        """
        Resume COP1.
        """
        req = cop1_pb2.ResumeRequest()
        url = f"/cop1/{self._instance}/{self._link}:resume"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def get_cop1_status(self) -> Cop1Status:
        """
        Retrieve the COP1 status.
        """
        response = self.ctx.get_proto(f"/cop1/{self._instance}/{self._link}/status")
        message = cop1_pb2.Cop1Status()
        message.ParseFromString(response.content)
        return Cop1Status(message)

    def create_cop1_subscription(
        self, on_data: Callable[[Cop1Status], None], timeout: float = 60
    ) -> Cop1Subscription:
        """
        Create a new subscription for receiving status of the COP1 link.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param on_data:
            Function that gets called on each :class:`.Cop1Status`.
        :param timeout:
            The amount of seconds to wait for the request to complete.
        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = cop1_pb2.SubscribeStatusRequest()
        options.instance = self._instance
        options.link = self._link

        manager = WebSocketSubscriptionManager(self.ctx, topic="cop1", options=options)

        # Represent subscription as a future
        subscription = Cop1Subscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_cop1_status, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription



import datetime
import functools
import os
import urllib.parse
from typing import Any, Callable, Iterable, Iterator, List, Mapping, Optional, Union
from urllib.parse import urlparse

from google.protobuf import timestamp_pb2
from yamcs.archive.client import ArchiveClient
from yamcs.core.auth import APIKeyCredentials, Credentials
from yamcs.core.context import Context
from yamcs.core.futures import WebSocketSubscriptionFuture
from yamcs.core.helpers import (
    delimit_protobuf,
    do_get,
    parse_server_time,
    to_server_time,
)
from yamcs.core.subscriptions import WebSocketSubscriptionManager
from yamcs.filetransfer.client import FileTransferClient
from yamcs.link.client import LinkClient
from yamcs.mdb.client import MDBClient
from yamcs.model import (
    AuthInfo,
    Event,
    Instance,
    InstanceTemplate,
    Link,
    LoadParameterValuesResult,
    Processor,
    RdbTablespace,
    ServerInfo,
    Service,
    UserInfo,
)
from yamcs.protobuf.archive import rocksdb_service_pb2
from yamcs.protobuf.auth import auth_pb2
from yamcs.protobuf.events import events_pb2, events_service_pb2
from yamcs.protobuf.iam import iam_pb2
from yamcs.protobuf.instances import instances_pb2, instances_service_pb2
from yamcs.protobuf.links import links_pb2
from yamcs.protobuf.processing import processing_pb2
from yamcs.protobuf.pvalue import pvalue_service_pb2
from yamcs.protobuf.server import server_service_pb2
from yamcs.protobuf.services import services_service_pb2
from yamcs.protobuf.time import time_service_pb2
from yamcs.storage.client import StorageClient
from yamcs.tco.client import TCOClient
from yamcs.timeline.client import TimelineClient
from yamcs.tmtc.client import ProcessorClient, _build_value_proto
from yamcs.tmtc.model import ValueUpdate


def _wrap_callback_parse_time_info(subscription, on_data, message):
    """
    Wraps a user callback to parse TimeInfo
    from a WebSocket data message
    """
    pb = timestamp_pb2.Timestamp()
    message.Unpack(pb)
    time = parse_server_time(pb)
    subscription._process(time)
    if on_data:
        on_data(time)


def _wrap_callback_parse_event(on_data, message):
    """
    Wraps a user callback to parse Events
    from a WebSocket data message
    """
    pb = events_pb2.Event()
    message.Unpack(pb)
    event = Event(pb)
    on_data(event)


def _wrap_callback_parse_link_event(subscription, on_data, message):
    """
    Wraps a user callback to parse Link[] updates
    from a WebSocket data message
    """
    pb = links_pb2.LinkEvent()
    message.Unpack(pb)
    links = [Link(link_pb) for link_pb in pb.links]
    subscription._process(links)
    if on_data:
        on_data(links)


class TimeSubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to time updates.

    A subscription object stores the last time info.
    """

    def __init__(self, manager):
        super(TimeSubscription, self).__init__(manager)

        self.time: Optional[datetime.datetime] = None
        """The last time info."""

    def _process(self, time):
        self.time = time


class LinkSubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to data link updates.

    A subscription object stores the last link info for
    each link.
    """

    def __init__(self, manager):
        super(LinkSubscription, self).__init__(manager)

        self._cache = {}
        """Link cache keyed by name."""

    def get_link(self, name: str) -> Optional[Link]:
        """
        Returns the latest link state.

        :param name:
            Identifying name of the data link
        """
        if name in self._cache:
            return self._cache[name]
        return None

    def list_links(self) -> List[Link]:
        """
        Returns a snapshot of all links.
        """
        return [self._cache[k] for k in self._cache]

    def _process(self, links: List[Link]):
        for link in links:
            self._cache[link.name] = link


class YamcsClient:
    """
    Client for accessing core Yamcs resources.
    """

    def __init__(
        self,
        address: str,
        *,
        tls: bool = False,
        tls_verify: Union[bool, str] = True,
        credentials: Optional[Credentials] = None,
        user_agent: Optional[str] = None,
        keep_alive: bool = True,
        on_token_update: Optional[Callable[[Credentials], None]] = None,
    ):
        """
        :param address:
            The address of Yamcs in the format 'hostname:port'
        :param tls:
            Whether TLS encryption is expected
        :param tls_verify:
            Whether server certificate verification is
            enabled (only applicable if ``tls=True``).
            As an alternative to a boolean value, this option
            may be set to a path containing the appropriate
            TLS CA certificate bundle.
        :param credentials:
            Credentials for when the server is secured
        :param user_agent:
            Optionally override the default user agent
        :param keep_alive:
            Automatically renew the client session. If disabled,
            the session will terminate after about 30 minutes of
            inactivity.

            This property is only considered when accessing a
            server that requires authentication.
        """

        # Allow server URLs.
        # Currently undocumented, but this is expected to become the
        # default, later on.
        if address.startswith("http://") or address.startswith("https://"):
            components = urlparse(address)
            tls = components.scheme == "https"
            address = components.netloc
            address += components.path

        self.ctx = Context(
            address=address,
            tls=tls,
            credentials=credentials,
            user_agent=user_agent,
            on_token_update=on_token_update,
            tls_verify=tls_verify,
            keep_alive=keep_alive,
        )

    @staticmethod
    def from_environment():
        """
        Create a :class:`.YamcsClient`, initialized from environment variables.

        This recognizes the following environment variables:

        ``YAMCS_URL``
            Yamcs server URL

        ``YAMCS_API_KEY``
            Yamcs API key (currently only assigned to script activities)
        """
        url = os.environ["YAMCS_URL"]

        credentials = None
        api_key = os.environ.get("YAMCS_API_KEY")
        if api_key:
            credentials = APIKeyCredentials(api_key)

        return YamcsClient(url, credentials=credentials)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def get_time(self, instance) -> Optional[datetime.datetime]:
        """
        Return the current mission time for the specified instance.
        """
        url = f"/instances/{instance}"
        response = self.ctx.get_proto(url)
        message = instances_pb2.YamcsInstance()
        message.ParseFromString(response.content)
        if message.HasField("missionTime"):
            return parse_server_time(message.missionTime)
        return None

    def get_server_info(self) -> ServerInfo:
        """
        Return general server info.
        """
        response = self.ctx.get_proto(path="")
        message = server_service_pb2.GetServerInfoResponse()
        message.ParseFromString(response.content)
        return ServerInfo(message)

    def get_auth_info(self) -> AuthInfo:
        """
        Returns general authentication information. This operation
        does not require authenticating and is useful to test
        if a server requires authentication or not.
        """
        response = do_get(
            self.ctx.session,
            self.ctx.auth_root,  # Full URL, so don't use get_proto
            headers={"Accept": "application/protobuf"},
        )
        message = auth_pb2.AuthInfo()
        message.ParseFromString(response.content)
        return AuthInfo(message)

    def get_user_info(self) -> UserInfo:
        """
        Get information on the authenticated user.
        """
        response = self.ctx.get_proto(path="/user")
        message = iam_pb2.UserInfo()
        message.ParseFromString(response.content)
        return UserInfo(message)

    def get_mdb(self, instance: str) -> MDBClient:
        """
        Return an object for working with the MDB of the specified instance.

        :param instance:
            A Yamcs instance name.
        """
        return MDBClient(self.ctx, instance)

    def get_archive(self, instance: str) -> ArchiveClient:
        """
        Return an object for working with the Archive of the specified instance.

        :param instance:
            A Yamcs instance name.
        """
        return ArchiveClient(self.ctx, instance)

    def get_file_transfer_client(self, instance: str) -> FileTransferClient:
        """
        Return an object for working with file transfers on a specified instance.

        :param instance:
            A Yamcs instance name.
        """
        return FileTransferClient(self.ctx, instance)

    def get_tco_client(self, instance: str, service: str) -> TCOClient:
        """
        Return an object for Time Correlation API calls on a specified service.

        :param instance:
            A Yamcs instance name.
        :param service:
            Target service name.
        """
        return TCOClient(self.ctx, instance, service)

    def get_storage_client(self) -> StorageClient:
        """
        Return an object for working with object storage
        """
        return StorageClient(self.ctx)

    def get_timeline_client(self, instance: str) -> TimelineClient:
        """
        Return an object for working with Yamcs timeline items

        :param instance:
            A Yamcs instance name.
        """
        return TimelineClient(self.ctx, instance)

    def create_instance(
        self,
        name: str,
        template: str,
        args: Optional[Mapping[str, Any]] = None,
        labels: Optional[Mapping[str, str]] = None,
    ):
        """
        Create a new instance based on an existing template. This method blocks
        until the instance is fully started.

        :param instance:
            A Yamcs instance name.
        :param template:
            The name of an existing template.
        """
        req = instances_service_pb2.CreateInstanceRequest()
        req.name = name
        req.template = template
        if args:
            for k in args:
                req.templateArgs[k] = args[k]
        if labels:
            for k in labels:
                req.labels[k] = labels[k]
        url = "/instances"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def list_instance_templates(self) -> Iterable[InstanceTemplate]:
        """
        List the available instance templates.
        """
        response = self.ctx.get_proto(path="/instance-templates")
        message = instances_service_pb2.ListInstanceTemplatesResponse()
        message.ParseFromString(response.content)
        templates = getattr(message, "templates")
        return iter([InstanceTemplate(template) for template in templates])

    def list_services(self, instance: str) -> Iterable[Service]:
        """
        List the services for an instance.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = f"/services/{instance}"
        response = self.ctx.get_proto(path=url)
        message = services_service_pb2.ListServicesResponse()
        message.ParseFromString(response.content)
        services = getattr(message, "services")
        return iter([Service(service) for service in services])

    def start_service(self, instance: str, service: str):
        """
        Starts a single service.

        :param instance:
            A Yamcs instance name.
        :param service:
            The name of the service.
        """
        url = f"/services/{instance}/{service}:start"
        self.ctx.post_proto(url)

    def stop_service(self, instance: str, service: str):
        """
        Stops a single service.

        :param instance:
            A Yamcs instance name.
        :param service:
            The name of the service.
        """
        url = f"/services/{instance}/{service}:stop"
        self.ctx.post_proto(url)

    def list_processors(self, instance: Optional[str] = None) -> Iterable[Processor]:
        """
        Lists the processors.

        Processors are returned in lexicographical order.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = "/processors"
        if instance:
            url += "?instance=" + instance
        response = self.ctx.get_proto(path=url)
        message = processing_pb2.ListProcessorsResponse()
        message.ParseFromString(response.content)
        processors = getattr(message, "processors")
        return iter([Processor(processor) for processor in processors])

    def get_processor(self, instance: str, processor: str) -> ProcessorClient:
        """
        Return an object for working with a specific Yamcs processor.

        :param instance:
            A Yamcs instance name.
        :param processor:
            A processor name within that instance.
        """
        return ProcessorClient(self.ctx, instance, processor)

    def delete_processor(self, instance: str, processor: str):
        """
        Delete a processor.

        :param instance:
            A Yamcs instance name.
        :param processor:
            A processor name within that instance.
        """
        url = f"/processors/{instance}/{processor}"
        self.ctx.delete_proto(url)

    def get_link(self, instance: str, link: str) -> LinkClient:
        """
        Return an object for working with a specific Yamcs link.

        :param instance:
            A Yamcs instance name.
        :param link:
            A link name within that instance.
        """
        return LinkClient(self.ctx, instance, link)

    def list_instances(self) -> Iterable[Instance]:
        """
        Lists the instances.

        Instances are returned in lexicographical order.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(path="/instances")
        message = instances_service_pb2.ListInstancesResponse()
        message.ParseFromString(response.content)
        instances = getattr(message, "instances")
        return iter([Instance(instance) for instance in instances])

    def start_instance(self, instance: str):
        """
        Starts a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:start"
        self.ctx.post_proto(url)

    def stop_instance(self, instance: str):
        """
        Stops a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:stop"
        self.ctx.post_proto(url)

    def restart_instance(self, instance: str):
        """
        Restarts a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:restart"
        self.ctx.post_proto(url)

    def list_links(self, instance: str) -> Iterable[Link]:
        """
        Lists the data links visible to this client.

        Data links are returned in random order.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(path="/links/" + instance)
        message = links_pb2.ListLinksResponse()
        message.ParseFromString(response.content)
        links = getattr(message, "links")
        return iter([Link(link) for link in links])

    def list_rdb_tablespaces(self) -> Iterable[RdbTablespace]:
        """
        Lists RocksDB tablespaces.
        """
        response = self.ctx.get_proto(path="/archive/rocksdb/tablespaces")
        message = rocksdb_service_pb2.ListRocksDbTablespacesResponse()
        message.ParseFromString(response.content)
        tablespaces = getattr(message, "tablespaces")
        return iter([RdbTablespace(tablespace) for tablespace in tablespaces])

    def compact_rdb_column_family(
        self,
        tablespace: str,
        cf: str,
        dbpath: Optional[str] = None,
    ):
        """
        Compact a RocksDB column family.

        :param tablespace:
            RocksDB tablespace name
        :param cf:
            Column family name
        :param dbpath:
            Optional path under the tablespace root.
        """
        req = rocksdb_service_pb2.CompactDatabaseRequest()
        req.cfname = cf
        encoded_name = urllib.parse.quote_plus(dbpath or "")
        url = f"/archive/rocksdb/{tablespace}/{encoded_name}:compact"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def send_event(
        self,
        instance: str,
        message: str,
        event_type: Optional[str] = None,
        time: Optional[datetime.datetime] = None,
        severity: Optional[str] = "info",
        source: Optional[str] = None,
        sequence_number: Optional[int] = None,
        extra: Optional[Mapping[str, str]] = None,
    ):
        """
        Post a new event.

        :param instance:
            A Yamcs instance name.
        :param message:
            Event message.
        :param event_type:
            Type of event.
        :param severity:
            The severity level of the event. One of ``info``,
            ``watch``, ``warning``, ``distress``, ``critical``
            or ``severe``. Defaults to ``info``.
        :param time:
            Time of the event. If unspecified, defaults to mission time.
        :param source:
            Source of the event. Useful for grouping events in the
            archive. When unset this defaults to ``User``.
        :param event_type:
            Event type.
        :param sequence_number:
            Sequence number of this event. This is used to
            determine unicity of events at the same time and
            coming from the same source. If not set Yamcs
            will automatically assign a sequential number as
            if every submitted event is unique.
        :param extra:
            Extra event properties.

            .. versionadded:: 1.8.4
               Compatible with Yamcs 5.7.3 onwards
        """
        req = events_service_pb2.CreateEventRequest()
        req.message = message
        req.severity = severity
        if event_type:
            req.type = event_type
        if time:
            req.time.MergeFrom(to_server_time(time))
        if source:
            req.source = source
        if sequence_number is not None:
            req.sequenceNumber = sequence_number
        if extra:
            for key in extra:
                req.extra[key] = extra[key]

        url = f"/archive/{instance}/events"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def load_parameter_values(
        self,
        instance: str,
        data: Iterator[Mapping[str, ValueUpdate]],
        stream: str = "pp_dump",
        chunk_size: int = 32 * 1024,
    ) -> LoadParameterValuesResult:
        """
        Load an indefinite amount of parameter values into Yamcs.

        .. versionadded:: 1.9.1
           Compatible with Yamcs 5.8.8 onwards

        :param instance:
            Yamcs instance name
        :param data:
            Generator that yields batches of parameter values,
            keyed by parameter name.
        :param stream:
            Stream where to send the parameters to.
        :param chunk_size:
            HTTP chunk size. Multiple updates are grouped in chunks
            of about this size.
        """

        def to_proto(generator: Iterator[Mapping[str, ValueUpdate]]):
            for values in generator:
                proto = pvalue_service_pb2.LoadParameterValuesRequest()
                for key in values:
                    value_update = values[key]
                    item = proto.values.add()
                    item.parameter = key
                    item.value.MergeFrom(_build_value_proto(value_update.value))
                    if value_update.generation_time:
                        item.generationTime.MergeFrom(
                            to_server_time(value_update.generation_time)
                        )
                    if value_update.expires_in is not None:
                        item.expiresIn = int(value_update.expires_in * 1000)

                yield proto

        generator = delimit_protobuf(to_proto(data), chunk_size=chunk_size)

        url = f"/parameter-values/{instance}/streams/{stream}:load"
        response = self.ctx.post_proto(url, data=generator)
        message = pvalue_service_pb2.LoadParameterValuesResponse()
        message.ParseFromString(response.content)
        return LoadParameterValuesResult(message)

    def create_link_subscription(
        self,
        instance: str,
        on_data: Optional[Callable[[List[Link]], None]] = None,
        timeout: float = 60,
    ) -> LinkSubscription:
        """
        Create a new subscription for receiving data link updates of an instance.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name.
        :param on_data:
            Function that gets called with :class:`.Link` updates. Each update
            contains state of all links.
        :param timeout:
            The amount of seconds to wait for the request to complete.
        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = links_pb2.SubscribeLinksRequest()
        options.instance = instance
        manager = WebSocketSubscriptionManager(self.ctx, topic="links", options=options)

        # Represent subscription as a future
        subscription = LinkSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_link_event, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_time_subscription(
        self,
        instance: str,
        on_data: Optional[Callable[[datetime.datetime], None]] = None,
        timeout: float = 60,
    ) -> TimeSubscription:
        """
        Create a new subscription for receiving time updates of an instance.
        Time updates are emitted at 1Hz.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name
        :param on_data:
            Function that gets called with :class:`~datetime.datetime` updates.
        :param timeout:
            The amount of seconds to wait for the request to complete.

        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = time_service_pb2.SubscribeTimeRequest()
        options.instance = instance
        manager = WebSocketSubscriptionManager(self.ctx, topic="time", options=options)

        # Represent subscription as a future
        subscription = TimeSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_time_info, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_event_subscription(
        self, instance: str, on_data: Callable[[Event], None], timeout: float = 60
    ) -> WebSocketSubscriptionFuture:
        """
        Create a new subscription for receiving events of an instance.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name
        :param on_data:
            Function that gets called on each :class:`.Event`.
        :param timeout:
            The amount of seconds to wait for the request to complete.
        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = events_service_pb2.SubscribeEventsRequest()
        options.instance = instance
        manager = WebSocketSubscriptionManager(
            self.ctx, topic="events", options=options
        )

        # Represent subscription as a future
        subscription = WebSocketSubscriptionFuture(manager)

        wrapped_callback = functools.partial(_wrap_callback_parse_event, on_data)

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def close(self):
        """Close this client session"""
        self.ctx.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/events/events_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from yamcs.api import annotations_pb2 as yamcs_dot_api_dot_annotations__pb2
from yamcs.api import httpbody_pb2 as yamcs_dot_api_dot_httpbody__pb2
from yamcs.protobuf.events import events_pb2 as yamcs_dot_protobuf_dot_events_dot_events__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/events/events_service.proto',
  package='yamcs.protobuf.events',
  syntax='proto2',
  serialized_options=b'\n\022org.yamcs.protobufB\022EventsServiceProtoP\001',
  serialized_pb=b'\n*yamcs/protobuf/events/events_service.proto\x12\x15yamcs.protobuf.events\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1byamcs/api/annotations.proto\x1a\x18yamcs/api/httpbody.proto\x1a\"yamcs/protobuf/events/events.proto\"\xe4\x01\n\x11ListEventsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0f\n\x03pos\x18\x02 \x01(\x03\x42\x02\x18\x01\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\r\n\x05order\x18\x04 \x01(\t\x12\x10\n\x08severity\x18\x05 \x01(\t\x12\x0e\n\x06source\x18\x06 \x03(\t\x12\x0c\n\x04next\x18\x07 \x01(\t\x12)\n\x05start\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\t\n\x01q\x18\n \x01(\t\"\x8e\x01\n\x12ListEventsResponse\x12/\n\x05\x65vent\x18\x01 \x03(\x0b\x32\x1c.yamcs.protobuf.events.EventB\x02\x18\x01\x12,\n\x06\x65vents\x18\x03 \x03(\x0b\x32\x1c.yamcs.protobuf.events.Event\x12\x19\n\x11\x63ontinuationToken\x18\x02 \x01(\t\"*\n\x16SubscribeEventsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"\x9c\x02\n\x12\x43reateEventRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x10\n\x08severity\x18\x04 \x01(\t\x12(\n\x04time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06source\x18\x06 \x01(\t\x12\x16\n\x0esequenceNumber\x18\x07 \x01(\x05\x12\x43\n\x05\x65xtra\x18\x08 \x03(\x0b\x32\x34.yamcs.protobuf.events.CreateEventRequest.ExtraEntry\x1a,\n\nExtraEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa9\x01\n\x13StreamEventsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06source\x18\x04 \x03(\t\x12\x10\n\x08severity\x18\x05 \x01(\t\x12\t\n\x01q\x18\x06 \x01(\t\"+\n\x17ListEventSourcesRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"?\n\x18ListEventSourcesResponse\x12\x12\n\x06source\x18\x01 \x03(\tB\x02\x18\x01\x12\x0f\n\x07sources\x18\x02 \x03(\t\"\xbc\x01\n\x13\x45xportEventsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06source\x18\x04 \x03(\t\x12\x10\n\x08severity\x18\x05 \x01(\t\x12\t\n\x01q\x18\x06 \x01(\t\x12\x11\n\tdelimiter\x18\x07 \x01(\t2\xbc\x06\n\tEventsApi\x12\x87\x01\n\nListEvents\x12(.yamcs.protobuf.events.ListEventsRequest\x1a).yamcs.protobuf.events.ListEventsResponse\"$\x8a\x92\x03 \n\x1e/api/archive/{instance}/events\x12\x7f\n\x0b\x43reateEvent\x12).yamcs.protobuf.events.CreateEventRequest\x1a\x1c.yamcs.protobuf.events.Event\"\'\x8a\x92\x03#\x1a\x1e/api/archive/{instance}/events:\x01*\x12\xa1\x01\n\x10ListEventSources\x12..yamcs.protobuf.events.ListEventSourcesRequest\x1a/.yamcs.protobuf.events.ListEventSourcesResponse\",\x8a\x92\x03(\n&/api/archive/{instance}/events/sources\x12\x90\x01\n\x0cStreamEvents\x12*.yamcs.protobuf.events.StreamEventsRequest\x1a\x1c.yamcs.protobuf.events.Event\"4\x8a\x92\x03\x30\x1a+/api/stream-archive/{instance}:streamEvents:\x01*0\x01\x12}\n\x0c\x45xportEvents\x12*.yamcs.protobuf.events.ExportEventsRequest\x1a\x13.yamcs.api.HttpBody\"*\x8a\x92\x03&\n$/api/archive/{instance}:exportEvents0\x01\x12n\n\x0fSubscribeEvents\x12-.yamcs.protobuf.events.SubscribeEventsRequest\x1a\x1c.yamcs.protobuf.events.Event\"\x0c\xda\x92\x03\x08\n\x06\x65vents0\x01\x42*\n\x12org.yamcs.protobufB\x12\x45ventsServiceProtoP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,yamcs_dot_api_dot_annotations__pb2.DESCRIPTOR,yamcs_dot_api_dot_httpbody__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_events_dot_events__pb2.DESCRIPTOR,])




_LISTEVENTSREQUEST = _descriptor.Descriptor(
  name='ListEventsRequest',
  full_name='yamcs.protobuf.events.ListEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.ListEventsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pos', full_name='yamcs.protobuf.events.ListEventsRequest.pos', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='yamcs.protobuf.events.ListEventsRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order', full_name='yamcs.protobuf.events.ListEventsRequest.order', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='yamcs.protobuf.events.ListEventsRequest.severity', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.events.ListEventsRequest.source', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next', full_name='yamcs.protobuf.events.ListEventsRequest.next', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.events.ListEventsRequest.start', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.events.ListEventsRequest.stop', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='q', full_name='yamcs.protobuf.events.ListEventsRequest.q', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=422,
)


_LISTEVENTSRESPONSE = _descriptor.Descriptor(
  name='ListEventsResponse',
  full_name='yamcs.protobuf.events.ListEventsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='event', full_name='yamcs.protobuf.events.ListEventsResponse.event', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='events', full_name='yamcs.protobuf.events.ListEventsResponse.events', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='continuationToken', full_name='yamcs.protobuf.events.ListEventsResponse.continuationToken', index=2,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=425,
  serialized_end=567,
)


_SUBSCRIBEEVENTSREQUEST = _descriptor.Descriptor(
  name='SubscribeEventsRequest',
  full_name='yamcs.protobuf.events.SubscribeEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.SubscribeEventsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=569,
  serialized_end=611,
)


_CREATEEVENTREQUEST_EXTRAENTRY = _descriptor.Descriptor(
  name='ExtraEntry',
  full_name='yamcs.protobuf.events.CreateEventRequest.ExtraEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='yamcs.protobuf.events.CreateEventRequest.ExtraEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='yamcs.protobuf.events.CreateEventRequest.ExtraEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=854,
  serialized_end=898,
)

_CREATEEVENTREQUEST = _descriptor.Descriptor(
  name='CreateEventRequest',
  full_name='yamcs.protobuf.events.CreateEventRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.CreateEventRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.events.CreateEventRequest.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='yamcs.protobuf.events.CreateEventRequest.message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='yamcs.protobuf.events.CreateEventRequest.severity', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='yamcs.protobuf.events.CreateEventRequest.time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.events.CreateEventRequest.source', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequenceNumber', full_name='yamcs.protobuf.events.CreateEventRequest.sequenceNumber', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extra', full_name='yamcs.protobuf.events.CreateEventRequest.extra', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CREATEEVENTREQUEST_EXTRAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=614,
  serialized_end=898,
)


_STREAMEVENTSREQUEST = _descriptor.Descriptor(
  name='StreamEventsRequest',
  full_name='yamcs.protobuf.events.StreamEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.StreamEventsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.events.StreamEventsRequest.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.events.StreamEventsRequest.stop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.events.StreamEventsRequest.source', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='yamcs.protobuf.events.StreamEventsRequest.severity', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='q', full_name='yamcs.protobuf.events.StreamEventsRequest.q', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=901,
  serialized_end=1070,
)


_LISTEVENTSOURCESREQUEST = _descriptor.Descriptor(
  name='ListEventSourcesRequest',
  full_name='yamcs.protobuf.events.ListEventSourcesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.ListEventSourcesRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1072,
  serialized_end=1115,
)


_LISTEVENTSOURCESRESPONSE = _descriptor.Descriptor(
  name='ListEventSourcesResponse',
  full_name='yamcs.protobuf.events.ListEventSourcesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.events.ListEventSourcesResponse.source', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sources', full_name='yamcs.protobuf.events.ListEventSourcesResponse.sources', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1117,
  serialized_end=1180,
)


_EXPORTEVENTSREQUEST = _descriptor.Descriptor(
  name='ExportEventsRequest',
  full_name='yamcs.protobuf.events.ExportEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.events.ExportEventsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.events.ExportEventsRequest.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.events.ExportEventsRequest.stop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.events.ExportEventsRequest.source', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='severity', full_name='yamcs.protobuf.events.ExportEventsRequest.severity', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='q', full_name='yamcs.protobuf.events.ExportEventsRequest.q', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delimiter', full_name='yamcs.protobuf.events.ExportEventsRequest.delimiter', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1183,
  serialized_end=1371,
)

_LISTEVENTSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTEVENTSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTEVENTSRESPONSE.fields_by_name['event'].message_type = yamcs_dot_protobuf_dot_events_dot_events__pb2._EVENT
_LISTEVENTSRESPONSE.fields_by_name['events'].message_type = yamcs_dot_protobuf_dot_events_dot_events__pb2._EVENT
_CREATEEVENTREQUEST_EXTRAENTRY.containing_type = _CREATEEVENTREQUEST
_CREATEEVENTREQUEST.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_CREATEEVENTREQUEST.fields_by_name['extra'].message_type = _CREATEEVENTREQUEST_EXTRAENTRY
_STREAMEVENTSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMEVENTSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPORTEVENTSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPORTEVENTSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['ListEventsRequest'] = _LISTEVENTSREQUEST
DESCRIPTOR.message_types_by_name['ListEventsResponse'] = _LISTEVENTSRESPONSE
DESCRIPTOR.message_types_by_name['SubscribeEventsRequest'] = _SUBSCRIBEEVENTSREQUEST
DESCRIPTOR.message_types_by_name['CreateEventRequest'] = _CREATEEVENTREQUEST
DESCRIPTOR.message_types_by_name['StreamEventsRequest'] = _STREAMEVENTSREQUEST
DESCRIPTOR.message_types_by_name['ListEventSourcesRequest'] = _LISTEVENTSOURCESREQUEST
DESCRIPTOR.message_types_by_name['ListEventSourcesResponse'] = _LISTEVENTSOURCESRESPONSE
DESCRIPTOR.message_types_by_name['ExportEventsRequest'] = _EXPORTEVENTSREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListEventsRequest = _reflection.GeneratedProtocolMessageType('ListEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.ListEventsRequest)
  })
_sym_db.RegisterMessage(ListEventsRequest)

ListEventsResponse = _reflection.GeneratedProtocolMessageType('ListEventsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSRESPONSE,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.ListEventsResponse)
  })
_sym_db.RegisterMessage(ListEventsResponse)

SubscribeEventsRequest = _reflection.GeneratedProtocolMessageType('SubscribeEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _SUBSCRIBEEVENTSREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.SubscribeEventsRequest)
  })
_sym_db.RegisterMessage(SubscribeEventsRequest)

CreateEventRequest = _reflection.GeneratedProtocolMessageType('CreateEventRequest', (_message.Message,), {

  'ExtraEntry' : _reflection.GeneratedProtocolMessageType('ExtraEntry', (_message.Message,), {
    'DESCRIPTOR' : _CREATEEVENTREQUEST_EXTRAENTRY,
    '__module__' : 'yamcs.protobuf.events.events_service_pb2'
    # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.CreateEventRequest.ExtraEntry)
    })
  ,
  'DESCRIPTOR' : _CREATEEVENTREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.CreateEventRequest)
  })
_sym_db.RegisterMessage(CreateEventRequest)
_sym_db.RegisterMessage(CreateEventRequest.ExtraEntry)

StreamEventsRequest = _reflection.GeneratedProtocolMessageType('StreamEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEVENTSREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.StreamEventsRequest)
  })
_sym_db.RegisterMessage(StreamEventsRequest)

ListEventSourcesRequest = _reflection.GeneratedProtocolMessageType('ListEventSourcesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSOURCESREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.ListEventSourcesRequest)
  })
_sym_db.RegisterMessage(ListEventSourcesRequest)

ListEventSourcesResponse = _reflection.GeneratedProtocolMessageType('ListEventSourcesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTEVENTSOURCESRESPONSE,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.ListEventSourcesResponse)
  })
_sym_db.RegisterMessage(ListEventSourcesResponse)

ExportEventsRequest = _reflection.GeneratedProtocolMessageType('ExportEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTEVENTSREQUEST,
  '__module__' : 'yamcs.protobuf.events.events_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.events.ExportEventsRequest)
  })
_sym_db.RegisterMessage(ExportEventsRequest)


DESCRIPTOR._options = None
_LISTEVENTSREQUEST.fields_by_name['pos']._options = None
_LISTEVENTSRESPONSE.fields_by_name['event']._options = None
_CREATEEVENTREQUEST_EXTRAENTRY._options = None
_LISTEVENTSOURCESRESPONSE.fields_by_name['source']._options = None

_EVENTSAPI = _descriptor.ServiceDescriptor(
  name='EventsApi',
  full_name='yamcs.protobuf.events.EventsApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1374,
  serialized_end=2202,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListEvents',
    full_name='yamcs.protobuf.events.EventsApi.ListEvents',
    index=0,
    containing_service=None,
    input_type=_LISTEVENTSREQUEST,
    output_type=_LISTEVENTSRESPONSE,
    serialized_options=b'\212\222\003 \n\036/api/archive/{instance}/events',
  ),
  _descriptor.MethodDescriptor(
    name='CreateEvent',
    full_name='yamcs.protobuf.events.EventsApi.CreateEvent',
    index=1,
    containing_service=None,
    input_type=_CREATEEVENTREQUEST,
    output_type=yamcs_dot_protobuf_dot_events_dot_events__pb2._EVENT,
    serialized_options=b'\212\222\003#\032\036/api/archive/{instance}/events:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='ListEventSources',
    full_name='yamcs.protobuf.events.EventsApi.ListEventSources',
    index=2,
    containing_service=None,
    input_type=_LISTEVENTSOURCESREQUEST,
    output_type=_LISTEVENTSOURCESRESPONSE,
    serialized_options=b'\212\222\003(\n&/api/archive/{instance}/events/sources',
  ),
  _descriptor.MethodDescriptor(
    name='StreamEvents',
    full_name='yamcs.protobuf.events.EventsApi.StreamEvents',
    index=3,
    containing_service=None,
    input_type=_STREAMEVENTSREQUEST,
    output_type=yamcs_dot_protobuf_dot_events_dot_events__pb2._EVENT,
    serialized_options=b'\212\222\0030\032+/api/stream-archive/{instance}:streamEvents:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='ExportEvents',
    full_name='yamcs.protobuf.events.EventsApi.ExportEvents',
    index=4,
    containing_service=None,
    input_type=_EXPORTEVENTSREQUEST,
    output_type=yamcs_dot_api_dot_httpbody__pb2._HTTPBODY,
    serialized_options=b'\212\222\003&\n$/api/archive/{instance}:exportEvents',
  ),
  _descriptor.MethodDescriptor(
    name='SubscribeEvents',
    full_name='yamcs.protobuf.events.EventsApi.SubscribeEvents',
    index=5,
    containing_service=None,
    input_type=_SUBSCRIBEEVENTSREQUEST,
    output_type=yamcs_dot_protobuf_dot_events_dot_events__pb2._EVENT,
    serialized_options=b'\332\222\003\010\n\006events',
  ),
])
_sym_db.RegisterServiceDescriptor(_EVENTSAPI)

DESCRIPTOR.services_by_name['EventsApi'] = _EVENTSAPI

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/yamcsManagement/yamcsManagement.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from yamcs.protobuf import yamcs_pb2 as yamcs_dot_protobuf_dot_yamcs__pb2
from yamcs.protobuf.services import services_pb2 as yamcs_dot_protobuf_dot_services_dot_services__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/yamcsManagement/yamcsManagement.proto',
  package='yamcs.protobuf.yamcsManagement',
  syntax='proto2',
  serialized_options=b'\n\022org.yamcs.protobufB\024YamcsManagementProtoP\001',
  serialized_pb=b'\n4yamcs/protobuf/yamcsManagement/yamcsManagement.proto\x12\x1eyamcs.protobuf.yamcsManagement\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ayamcs/protobuf/yamcs.proto\x1a&yamcs/protobuf/services/services.proto\"\xb6\x04\n\rProcessorInfo\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x0c\n\x04spec\x18\x04 \x01(\t\x12\x0f\n\x07\x63reator\x18\x05 \x01(\t\x12\x11\n\thasAlarms\x18\x06 \x01(\x08\x12\x15\n\rhasCommanding\x18\x07 \x01(\x08\x12\x34\n\x05state\x18\x08 \x01(\x0e\x32%.yamcs.protobuf.services.ServiceState\x12\x34\n\rreplayRequest\x18\t \x01(\x0b\x32\x1d.yamcs.protobuf.ReplayRequest\x12=\n\x0breplayState\x18\n \x01(\x0e\x32(.yamcs.protobuf.ReplayStatus.ReplayState\x12\x36\n\x08services\x18\x10 \x03(\x0b\x32$.yamcs.protobuf.services.ServiceInfo\x12\x12\n\npersistent\x18\x11 \x01(\x08\x12(\n\x04time\x18\x12 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06replay\x18\x13 \x01(\x08\x12\x1d\n\x15\x63heckCommandClearance\x18\x14 \x01(\x08\x12\x11\n\tprotected\x18\x15 \x01(\x08\x12K\n\x0f\x61\x63knowledgments\x18\x16 \x03(\x0b\x32\x32.yamcs.protobuf.yamcsManagement.AcknowledgmentInfo\"7\n\x12\x41\x63knowledgmentInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"\x98\x02\n\x0cTmStatistics\x12\x12\n\npacketName\x18\x01 \x01(\t\x12\x15\n\rqualifiedName\x18\t \x01(\t\x12\x17\n\x0freceivedPackets\x18\x02 \x01(\x03\x12 \n\x18subscribedParameterCount\x18\x06 \x01(\x05\x12\x30\n\x0clastReceived\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0elastPacketTime\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\npacketRate\x18\x0c \x01(\x03\x12\x10\n\x08\x64\x61taRate\x18\r \x01(\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t\"\xb3\x01\n\nStatistics\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x07 \x01(\t\x12=\n\x07tmstats\x18\x03 \x03(\x0b\x32,.yamcs.protobuf.yamcsManagement.TmStatistics\x12/\n\x0blastUpdated\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampJ\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\x8b\x02\n\x1aProcessorManagementRequest\x12W\n\toperation\x18\x01 \x01(\x0e\x32\x44.yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.Operation\x12\x10\n\x08instance\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0e\n\x06\x63onfig\x18\x05 \x01(\t\x12\x19\n\npersistent\x18\x07 \x01(\x08:\x05\x66\x61lse\";\n\tOperation\x12\x14\n\x10\x43REATE_PROCESSOR\x10\x00\x12\x18\n\x14\x43ONNECT_TO_PROCESSOR\x10\x01\"\x85\x02\n\x10ProcessorRequest\x12M\n\toperation\x18\x01 \x01(\x0e\x32:.yamcs.protobuf.yamcsManagement.ProcessorRequest.Operation\x12\x10\n\x08instance\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x10\n\x08seekTime\x18\x04 \x01(\x03\x12\x30\n\x0breplaySpeed\x18\x05 \x01(\x0b\x32\x1b.yamcs.protobuf.ReplaySpeed\">\n\tOperation\x12\t\n\x05PAUSE\x10\x02\x12\n\n\x06RESUME\x10\x03\x12\x08\n\x04SEEK\x10\x04\x12\x10\n\x0c\x43HANGE_SPEED\x10\x05\"\x9d\x01\n\x0bStreamEvent\x12>\n\x04type\x18\x01 \x01(\x0e\x32\x30.yamcs.protobuf.yamcsManagement.StreamEvent.Type\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tdataCount\x18\x03 \x01(\x03\"-\n\x04Type\x12\x0b\n\x07\x43REATED\x10\x01\x12\x0b\n\x07\x44\x45LETED\x10\x02\x12\x0b\n\x07UPDATED\x10\x03\x42,\n\x12org.yamcs.protobufB\x14YamcsManagementProtoP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_yamcs__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_services_dot_services__pb2.DESCRIPTOR,])



_PROCESSORMANAGEMENTREQUEST_OPERATION = _descriptor.EnumDescriptor(
  name='Operation',
  full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.Operation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CREATE_PROCESSOR', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONNECT_TO_PROCESSOR', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1489,
  serialized_end=1548,
)
_sym_db.RegisterEnumDescriptor(_PROCESSORMANAGEMENTREQUEST_OPERATION)

_PROCESSORREQUEST_OPERATION = _descriptor.EnumDescriptor(
  name='Operation',
  full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.Operation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PAUSE', index=0, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESUME', index=1, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SEEK', index=2, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHANGE_SPEED', index=3, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1750,
  serialized_end=1812,
)
_sym_db.RegisterEnumDescriptor(_PROCESSORREQUEST_OPERATION)

_STREAMEVENT_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='yamcs.protobuf.yamcsManagement.StreamEvent.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CREATED', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DELETED', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATED', index=2, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1927,
  serialized_end=1972,
)
_sym_db.RegisterEnumDescriptor(_STREAMEVENT_TYPE)


_PROCESSORINFO = _descriptor.Descriptor(
  name='ProcessorInfo',
  full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='spec', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.spec', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='creator', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.creator', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasAlarms', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.hasAlarms', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasCommanding', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.hasCommanding', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.state', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replayRequest', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.replayRequest', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replayState', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.replayState', index=9,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='services', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.services', index=10,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='persistent', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.persistent', index=11,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.time', index=12,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replay', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.replay', index=13,
      number=19, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='checkCommandClearance', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.checkCommandClearance', index=14,
      number=20, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protected', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.protected', index=15,
      number=21, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgments', full_name='yamcs.protobuf.yamcsManagement.ProcessorInfo.acknowledgments', index=16,
      number=22, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=756,
)


_ACKNOWLEDGMENTINFO = _descriptor.Descriptor(
  name='AcknowledgmentInfo',
  full_name='yamcs.protobuf.yamcsManagement.AcknowledgmentInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.yamcsManagement.AcknowledgmentInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='yamcs.protobuf.yamcsManagement.AcknowledgmentInfo.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=813,
)


_TMSTATISTICS = _descriptor.Descriptor(
  name='TmStatistics',
  full_name='yamcs.protobuf.yamcsManagement.TmStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='packetName', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.packetName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='qualifiedName', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.qualifiedName', index=1,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receivedPackets', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.receivedPackets', index=2,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subscribedParameterCount', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.subscribedParameterCount', index=3,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastReceived', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.lastReceived', index=4,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastPacketTime', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.lastPacketTime', index=5,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='packetRate', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.packetRate', index=6,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataRate', full_name='yamcs.protobuf.yamcsManagement.TmStatistics.dataRate', index=7,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=816,
  serialized_end=1096,
)


_STATISTICS = _descriptor.Descriptor(
  name='Statistics',
  full_name='yamcs.protobuf.yamcsManagement.Statistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.yamcsManagement.Statistics.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.yamcsManagement.Statistics.processor', index=1,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tmstats', full_name='yamcs.protobuf.yamcsManagement.Statistics.tmstats', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastUpdated', full_name='yamcs.protobuf.yamcsManagement.Statistics.lastUpdated', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1099,
  serialized_end=1278,
)


_PROCESSORMANAGEMENTREQUEST = _descriptor.Descriptor(
  name='ProcessorManagementRequest',
  full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.operation', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.instance', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.config', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='persistent', full_name='yamcs.protobuf.yamcsManagement.ProcessorManagementRequest.persistent', index=5,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PROCESSORMANAGEMENTREQUEST_OPERATION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1281,
  serialized_end=1548,
)


_PROCESSORREQUEST = _descriptor.Descriptor(
  name='ProcessorRequest',
  full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.operation', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.instance', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seekTime', full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.seekTime', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replaySpeed', full_name='yamcs.protobuf.yamcsManagement.ProcessorRequest.replaySpeed', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PROCESSORREQUEST_OPERATION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1551,
  serialized_end=1812,
)


_STREAMEVENT = _descriptor.Descriptor(
  name='StreamEvent',
  full_name='yamcs.protobuf.yamcsManagement.StreamEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.yamcsManagement.StreamEvent.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.yamcsManagement.StreamEvent.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataCount', full_name='yamcs.protobuf.yamcsManagement.StreamEvent.dataCount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STREAMEVENT_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1815,
  serialized_end=1972,
)

_PROCESSORINFO.fields_by_name['state'].enum_type = yamcs_dot_protobuf_dot_services_dot_services__pb2._SERVICESTATE
_PROCESSORINFO.fields_by_name['replayRequest'].message_type = yamcs_dot_protobuf_dot_yamcs__pb2._REPLAYREQUEST
_PROCESSORINFO.fields_by_name['replayState'].enum_type = yamcs_dot_protobuf_dot_yamcs__pb2._REPLAYSTATUS_REPLAYSTATE
_PROCESSORINFO.fields_by_name['services'].message_type = yamcs_dot_protobuf_dot_services_dot_services__pb2._SERVICEINFO
_PROCESSORINFO.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROCESSORINFO.fields_by_name['acknowledgments'].message_type = _ACKNOWLEDGMENTINFO
_TMSTATISTICS.fields_by_name['lastReceived'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TMSTATISTICS.fields_by_name['lastPacketTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STATISTICS.fields_by_name['tmstats'].message_type = _TMSTATISTICS
_STATISTICS.fields_by_name['lastUpdated'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROCESSORMANAGEMENTREQUEST.fields_by_name['operation'].enum_type = _PROCESSORMANAGEMENTREQUEST_OPERATION
_PROCESSORMANAGEMENTREQUEST_OPERATION.containing_type = _PROCESSORMANAGEMENTREQUEST
_PROCESSORREQUEST.fields_by_name['operation'].enum_type = _PROCESSORREQUEST_OPERATION
_PROCESSORREQUEST.fields_by_name['replaySpeed'].message_type = yamcs_dot_protobuf_dot_yamcs__pb2._REPLAYSPEED
_PROCESSORREQUEST_OPERATION.containing_type = _PROCESSORREQUEST
_STREAMEVENT.fields_by_name['type'].enum_type = _STREAMEVENT_TYPE
_STREAMEVENT_TYPE.containing_type = _STREAMEVENT
DESCRIPTOR.message_types_by_name['ProcessorInfo'] = _PROCESSORINFO
DESCRIPTOR.message_types_by_name['AcknowledgmentInfo'] = _ACKNOWLEDGMENTINFO
DESCRIPTOR.message_types_by_name['TmStatistics'] = _TMSTATISTICS
DESCRIPTOR.message_types_by_name['Statistics'] = _STATISTICS
DESCRIPTOR.message_types_by_name['ProcessorManagementRequest'] = _PROCESSORMANAGEMENTREQUEST
DESCRIPTOR.message_types_by_name['ProcessorRequest'] = _PROCESSORREQUEST
DESCRIPTOR.message_types_by_name['StreamEvent'] = _STREAMEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProcessorInfo = _reflection.GeneratedProtocolMessageType('ProcessorInfo', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSORINFO,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.ProcessorInfo)
  })
_sym_db.RegisterMessage(ProcessorInfo)

AcknowledgmentInfo = _reflection.GeneratedProtocolMessageType('AcknowledgmentInfo', (_message.Message,), {
  'DESCRIPTOR' : _ACKNOWLEDGMENTINFO,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.AcknowledgmentInfo)
  })
_sym_db.RegisterMessage(AcknowledgmentInfo)

TmStatistics = _reflection.GeneratedProtocolMessageType('TmStatistics', (_message.Message,), {
  'DESCRIPTOR' : _TMSTATISTICS,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.TmStatistics)
  })
_sym_db.RegisterMessage(TmStatistics)

Statistics = _reflection.GeneratedProtocolMessageType('Statistics', (_message.Message,), {
  'DESCRIPTOR' : _STATISTICS,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.Statistics)
  })
_sym_db.RegisterMessage(Statistics)

ProcessorManagementRequest = _reflection.GeneratedProtocolMessageType('ProcessorManagementRequest', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSORMANAGEMENTREQUEST,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.ProcessorManagementRequest)
  })
_sym_db.RegisterMessage(ProcessorManagementRequest)

ProcessorRequest = _reflection.GeneratedProtocolMessageType('ProcessorRequest', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSORREQUEST,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.ProcessorRequest)
  })
_sym_db.RegisterMessage(ProcessorRequest)

StreamEvent = _reflection.GeneratedProtocolMessageType('StreamEvent', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEVENT,
  '__module__' : 'yamcs.protobuf.yamcsManagement.yamcsManagement_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.yamcsManagement.StreamEvent)
  })
_sym_db.RegisterMessage(StreamEvent)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/actions/actions.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from yamcs.protobuf.config import config_pb2 as yamcs_dot_protobuf_dot_config_dot_config__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/actions/actions.proto',
  package='yamcs.protobuf.actions',
  syntax='proto2',
  serialized_options=b'\n\032org.yamcs.protobuf.actionsB\014ActionsProtoP\001',
  serialized_pb=b'\n$yamcs/protobuf/actions/actions.proto\x12\x16yamcs.protobuf.actions\x1a\"yamcs/protobuf/config/config.proto\"\x87\x01\n\nActionInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05label\x18\x02 \x01(\t\x12\r\n\x05style\x18\x03 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12\x0f\n\x07\x63hecked\x18\x05 \x01(\x08\x12-\n\x04spec\x18\x06 \x01(\x0b\x32\x1f.yamcs.protobuf.config.SpecInfoB,\n\x1aorg.yamcs.protobuf.actionsB\x0c\x41\x63tionsProtoP\x01'
  ,
  dependencies=[yamcs_dot_protobuf_dot_config_dot_config__pb2.DESCRIPTOR,])




_ACTIONINFO = _descriptor.Descriptor(
  name='ActionInfo',
  full_name='yamcs.protobuf.actions.ActionInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='yamcs.protobuf.actions.ActionInfo.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='yamcs.protobuf.actions.ActionInfo.label', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='style', full_name='yamcs.protobuf.actions.ActionInfo.style', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enabled', full_name='yamcs.protobuf.actions.ActionInfo.enabled', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='checked', full_name='yamcs.protobuf.actions.ActionInfo.checked', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='spec', full_name='yamcs.protobuf.actions.ActionInfo.spec', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=101,
  serialized_end=236,
)

_ACTIONINFO.fields_by_name['spec'].message_type = yamcs_dot_protobuf_dot_config_dot_config__pb2._SPECINFO
DESCRIPTOR.message_types_by_name['ActionInfo'] = _ACTIONINFO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ActionInfo = _reflection.GeneratedProtocolMessageType('ActionInfo', (_message.Message,), {
  'DESCRIPTOR' : _ACTIONINFO,
  '__module__' : 'yamcs.protobuf.actions.actions_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.actions.ActionInfo)
  })
_sym_db.RegisterMessage(ActionInfo)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from typing import IO, Any, Iterable, Mapping, Optional, Union

from yamcs.core.context import Context
from yamcs.core.exceptions import NotFound
from yamcs.protobuf.buckets import buckets_pb2
from yamcs.storage.model import Bucket, ObjectListing


class StorageClient:
    """
    Client for working with buckets and objects managed by Yamcs.
    """

    def __init__(self, ctx: Context):
        super(StorageClient, self).__init__()
        self.ctx = ctx

    def list_buckets(self) -> Iterable[Bucket]:
        """
        List the buckets.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(path="/storage/buckets")
        message = buckets_pb2.ListBucketsResponse()
        message.ParseFromString(response.content)
        buckets = getattr(message, "buckets")
        return iter([Bucket(bucket, self) for bucket in buckets])

    def get_bucket(self, name: str, create=False) -> Bucket:
        """
        Get a specific bucket.

        :param name:
            The bucket name.
        :param create:
            If specified, create the bucket if it does not yet exist.
        """
        url = "/storage/buckets/" + name

        if create:
            try:
                response = self.ctx.get_proto(path=url)
            except NotFound:
                self.create_bucket(name)
                response = self.ctx.get_proto(path=url)
        else:
            response = self.ctx.get_proto(path=url)
        message = buckets_pb2.BucketInfo()
        message.ParseFromString(response.content)
        return Bucket(message, self)

    def list_objects(
        self,
        bucket_name: str,
        prefix: Optional[str] = None,
        delimiter: Optional[str] = None,
    ) -> ObjectListing:
        """
        List the objects for a bucket.

        :param bucket_name:
            The name of the bucket.
        :param prefix:
            If specified, only objects that start with this
            prefix are listed.
        :param delimiter:
            If specified, return only objects whose name
            do not contain the delimiter after the prefix.
            For the other objects, the response contains
            (in the prefix response parameter) the name
            truncated after the delimiter. Duplicates are
            omitted.
        """
        url = f"/storage/buckets/{bucket_name}/objects"
        params = {}
        if prefix is not None:
            params["prefix"] = prefix
        if delimiter is not None:
            params["delimiter"] = delimiter
        response = self.ctx.get_proto(path=url, params=params)
        message = buckets_pb2.ListObjectsResponse()
        message.ParseFromString(response.content)
        return ObjectListing(message, bucket_name, self)

    def create_bucket(self, bucket_name: str):
        """
        Create a new bucket.

        :param bucket_name:
            The name of the bucket.
        """
        req = buckets_pb2.CreateBucketRequest()
        req.name = bucket_name
        url = "/storage/buckets"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def remove_bucket(self, bucket_name: str):
        """
        Remove a bucket.

        :param bucket_name:
            The name of the bucket.
        """
        url = f"/storage/buckets/{bucket_name}"
        self.ctx.delete_proto(url)

    def download_object(self, bucket_name: str, object_name: str) -> bytes:
        """
        Download an object.

        :param bucket_name:
            The name of the bucket.
        :param object_name:
            The object to fetch.
        """
        url = f"/storage/buckets/{bucket_name}/objects/{object_name}"
        response = self.ctx.get_proto(path=url)
        return response.content

    def upload_object(
        self,
        bucket_name: str,
        object_name: str,
        file_obj: Union[str, IO],
        content_type: Optional[str] = None,
        metadata: Optional[Mapping[str, str]] = None,
    ):
        """
        Upload an object to a bucket.

        :param bucket_name:
            The name of the bucket.
        :param object_name:
            The target name of the object.
        :param file_obj:
            The file (or file-like object) to upload.
        :param content_type:
            The content type associated to this object.
            This is mainly useful when accessing an object
            directly via a web browser. If unspecified, a
            content type *may* be automatically derived
            from the specified ``file_obj``.
        :param metadata:
            Optional metadata associated to this object.
        """
        url = f"/storage/buckets/{bucket_name}/objects/{object_name}"
        if content_type:
            files: Any = {object_name: (object_name, file_obj, content_type)}
        else:
            files: Any = {object_name: (object_name, file_obj)}
        if metadata:
            for k, v in metadata.items():
                files[k] = (None, v)
        self.ctx.request(path=url, method="post", files=files)

    def remove_object(self, bucket_name: str, object_name: str):
        """
        Remove an object from a bucket.

        :param bucket_name:
            The name of the bucket.
        :param object_name:
            The object to remove.
        """
        url = f"/storage/buckets/{bucket_name}/objects/{object_name}"
        self.ctx.delete_proto(url)
